%%This file runs the choke control case with the controller described in
%%the thesis. No observer is used for the flow, this is taken directly from
%%the simulated response. For the use of a observer, see the IRISDrill
%%control cases.
%%Written in 2011 by Ingar Landet

clc
clear all
%Global variables to communicate with ode solver. Bad programming, but
%works for the purpose
global p_cReal p_bReal w1Real omega P z_c xDot rho_in q_p q_bpp v_d geometry q_c;

dT = 0.05;                     % [s]       
N = 5;                            %  # cells in both drill string and annulus
v_d = 0;
q0 = 0*0.001/60;

%Heave simulation, read from logged Ullrigg data
load('Ullrigg_Eval05_20091126_1328.mat');
interval = 960:1200;
h_b = LOG.h_blkRig(interval);
v_EstUllRigg = LOG.v_blk(interval);
v_b = zeros(size(h_b));
p_ref = LOG.pRef_c(interval);
z_choke = LOG.z_c(interval);
q_pump = LOG.q_bpp(interval)*0.001/60;%q_pump = LOG.omega_p(interval)*16.7*0.001/60;
q_choke = LOG.q_c(interval)*0.001/60;
% Remove NaN-values because of lost samples
for i =1:length(h_b)
    if(isnan(h_b(i)))
        h_b(i) = h_b(i-1);
    end
    if(isnan(v_EstUllRigg(i)))
        v_EstUllRigg(i) = v_EstUllRigg(i-1);
    end
    if(isnan(q_choke(i)))
        q_choke(i) = q_choke(i-1);
    end
    if(isnan(z_choke(i)))
        z_choke(i) = z_choke(i-1);
    end
    if(isnan(q_pump(i)))
        q_pump(i) = q_pump(i-1);
    end
    if(isnan(p_ref(i)))
        p_ref(i) = p_ref(i-1);
    end
    if i > 1
        v_b(i) = (h_b(i) - h_b(i-1));
    end
end

%Filter to find drill string velocity, then time shift to eliminate lag
[B A] = butter(2,0.4,'low');
v_b = filter(B,A,v_b);
v_b = [v_b(1:4); v_b(8:end); v_b(end-4:end)];


%%Parameters
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1990.99;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1540;
geometry = findAreasAndVolumes(P);

%% Initial states
p_a0 = zeros(N,1);
p_ref = 240*10^5;
p_a0(1) = p_ref - 1420*9.81*P.h_dh/(2*N);
for i = 2:N
   p_a0(i) = p_a0(i-1) - 1420*9.81*P.h_dh/N; 
end
p_cRef = 40*10^5;%p_a0(N)-1420*9.81*P.h_dh/(2*N);
q_a0 = zeros(N-1,1);
x0 = [p_a0;q_a0];
z_c = 34.5;
xDot = zeros(2*N-1,1);            % allocate memory once, for speed

simPoints = 3000;
YExtended = zeros(2*N-1, simPoints);
q_choke = zeros(1,simPoints);
q_heave = zeros(1,simPoints);

y = x0';
q_p = 0;
q_c = q0;
q_bpp = 1500*0.001/60;


%parameters for controller
param.b0 = 35;
param.beta = 14000*10^5;
param.A = sum(geometry.A_a)/N;
param.l = geometry.l_d(1)*N/(N-0.5);
param.k_c = 545;
param.k_i = 0;
param.k_p = 0.01;
param.dT = dT;
param.timeStep = dT;
param.v_d = 0;
param.x_d = 0;
param.Lp = 1;
param.Li = 0.1;

% Extra for feedforward controller:
param.mju = 0.1257;
param.a = 0.49;
param.b = 0.99;
param.L = P.L;
param.rho = rho_in;
param.g = 9.81;
param.dh = P.h_dh;
param.v_dDot = 0;
param.v_dDotDot = 0;
param.d_a = 0.2304;
param.do = 0.1372;
param.l1 = 1.5;
param.l2 = 1.5;
param.l3 = 1.5;
param.omega = 2*pi/12;


controlZC = zeros(1, simPoints);
controlPump = zeros(1, simPoints);
pcArr = zeros(1,simPoints);
pbArr = zeros(1,simPoints);

qerr = 0;
p_hat = 5*10^5;

%Choke characteristics
Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
%g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );
p0 = 10^5;
zPI = 0;

for i = 1:simPoints
    p_c1 = getPc(y,P);
    p_cReal = p_c1;
    p_b1 = getPb(y,P);
    p_bReal = p_b1;
    omega = param.omega;
   
    if( i > 50)
        k = ceil((i-50)*dT); 
        A = 1;
%         v_d = -omega*A*sin(omega*i*dT);
        v_d = v_b(k);
        x_d = A*cos(omega*i*dT);
        w1Real = x_d;
        % Calculate control signal
        if mod(i,1) == 0

%             [z_c zPI] = paperFeedbackNoForward(y(end),q_bpp,p_c1,param,p_cRef,zPI);
            [z_c zPI q_bpp] = useQBPP(y(end),q_bpp,p_c1,param,p_cRef,zPI);
%             [z_c p_hat qerr] = statoilController(q_c,q_bpp,p_c1,param,Upi.p_cRef,p_hat,qerr);
        end

        
    end
    % Log variables
    q_choke(i) = q_c;
    q_heave(i) = v_d*pi*param.do^2/4;
    controlZC(i) = z_c;
    controlPump(i) = q_bpp;
    pcArr(i) = p_c1;
    pbArr(i) = p_b1;

    % run ode solver
    y = ode4(@fAnnulusHydr,[0 dT],y);
    y = y';
    y = y(:,end);
    YExtended(:,i) = y;
    if (mod(i,100) == 0)
        
        i/simPoints
    end
    
    
end
%% plot
subplot(2,2,1);
t=dT:dT:simPoints*dT;
plot(t,pbArr*10^-5);
xlabel('time [s]');
ylabel('p_b_i_t [bar]');
subplot(2,2,2);
plot(t,pcArr*10^-5);
xlabel('time [s]');
ylabel('p_c [bar]');
subplot(2,2,3);
plot(controlZC(1:20:end));
xlabel('time [s]');
ylabel('u_c [%]');
subplot(2,2,4);
plot(t,A*cos(omega*t));
xlabel('time [s]');
ylabel('x_d [m]');
