%%This file runs the heave compensation case on the IRISDrill simulator
%%using the internal model controller with feedback linearization for 
%%bottom hole pressure control.
%%Written in 2011 by Ingar Landet
clc
clear all;
global z_c  q_bpp v_d geometry xDot pbReal pcReal;

%% Initialize variables
Tstop= 2000;
FlowrateIn=zeros(Tstop,1);
AnnulusChokeOpening=zeros(Tstop,1);
ChokeLineChokeOpening=zeros(Tstop,1);
AnnulusPressure=zeros(Tstop,1);
ChokeLinePressure=zeros(Tstop,1);
ChokeLinePressure2 = zeros(Tstop,1);
StandpipePressure=zeros(Tstop,1);
BottomholePressure=zeros(Tstop,1);
BottomholePressure2 = zeros(Tstop,1);
AnnulusFlowrateOut=zeros(Tstop,1);
ChokeLineFlowrateOut=zeros(Tstop,1);
DensityIn=zeros(Tstop,1);
DensityOut=zeros(Tstop,1);
DensityAnnulus=zeros(Tstop,1);
ChokePressure= zeros(Tstop,1);
TopOfStringVelocity=zeros(Tstop,1);
qbpp = zeros(Tstop,1);
dT=0.05;
t=1.00;
F_run=true;
counter=1.00;

%% Initialize drillingcalculator
md=IRISDrill();
md.loadWellbore('U2262bar.binaryWellbore'); %Ullrigg well U2 data
md.initialize();
md.calc.StepDuration = dT;
md.calc.ChokeOpening = 1;
md.calc.ChokeLineChokeOpening = 1;
md.calc.FlowRateIn=0/60000;  % u1
md.calc.ChokePumpFlowRateIn=3000/60000; % u3 Flowrate for Choke Line Pump 
md.calc.DensityIn=1475; % u4
md.calc.ChokePumpDensityIn=1475; % u6 - choke pump fluid density  [kg/m3]
md.calc.Step();
md.calc.ChokeOpening = 0.3; % Control Choke, used as control input
md.calc.ChokeLineChokeOpening = 10^-4;  % Kill line choke, not used
t_cpu=0;
n = 1;



%% Initialize IMC controller
Q0 = 5;
Q1 = 5;
s = tf('s');
p_ref = 262*10^5;
% Use 1 single frequency
cSys = Q1 + Q1/s + Q0*s/(s^2+(pi/6)^2);

% Initial conditions for the controller
xW = [0 0 0];
array = zeros(1,100);
for i = 1:99
    t = [0:0.5:1];
    e = 10^-2*(-0.3-array(i));
    [Y,t,X] = lsim(ss(cSys),e*ones(size(t)),t,xW);
    xW = X(end,:);
    array(i+1) = Y(end);
end

K = [6 -2.5*10^8 6 -2.5*10^8 6 -2.5*10^8 6 -2.5*10^8 10];

% Choke characteristics
Z_c = (1/100)*[0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );

%% Initialize observer
% Parameters
N=5;
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1900;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1702;
geometry = findAreasAndVolumes(P);
YExtended = zeros(2*N-1, Tstop);
z_c = md.calc.ChokeOpening;
q_bpp = md.calc.ChokePumpFlowRateIn;
v_d = 0;
p_a0 = zeros(N,1);
p_c = 10^5;
p_a0(1) = p_c + 1475*9.81*P.h_dh;
for i = 2:N
   p_a0(i) = p_a0(i-1) - 1475*9.81*P.h_dh/(N-1); 
end
q_a0 = zeros(N-1,1);
y = [p_a0;q_a0];
xDot = zeros(2*N-1,1);            % allocate memory once, for speed

p0 = 1.0131*10^5;
x_d = 0;
k_c=2900;
p2a = 5.0909e+04;
k1 = (1/(N-0.5))*p2a;
param.rho = 1475;
param.A = sum(geometry.A_a)/N;
param.l = geometry.l_d(1);
param.dh = P.h_dh/4;
param.beta = 30000*10^5;

% Reference pressures (for zero flow)
p_ref2 = p_ref - k1 - param.rho*9.81*param.dh;
p_ref3 = p_ref2 - k1 - param.rho*9.81*param.dh;
p_ref4 = p_ref3 - k1 - param.rho*9.81*param.dh;
p_ref5 = p_ref4 - k1 - param.rho*9.81*param.dh;

beta1 = param.beta/(param.l*param.A);

%% Run to stabilize pressures
md.calc.StepDuration = 1;
for i = 1:300
    % Step simulator
    md.calc.Step();
    % Read pressure measurements
    pbReal = md.calc.CalculatedDownholePressure;
    pcReal = md.calc.CalculatedChokePressure;
    % Set global variables for observer
    z_c = md.calc.ChokeOpening;
    q_bpp = md.calc.ChokePumpFlowRateIn;
    v_d = 0;
    % Run observer
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
end
% Run control case
md.calc.StepDuration = dT;
while F_run
    tic	% Start timer
    % Move drillstring
    md.calc.TopOfStringVelocity = -2*pi/12*sin(2*pi*(n+i)*dT/12);
    % Run controller
    t = [0 dT];
    e = p_ref-md.calc.CalculatedDownholePressure;
    [Y,t,X] = lsim(ss(cSys),e*ones(size(t)),t,xW);
    xW = X(end,:);
    y_err = [p_ref5-y(5),-y(9),p_ref4-y(4),-y(8),p_ref3-y(3),-y(7),p_ref2-y(2),-y(6),p_ref-y(1)]';
    r = K*y_err + Y(end);
    u = (md.calc.ChokePumpFlowRateIn - r/beta1)/(k_c*sqrt(1400/1475)*sqrt((pcReal-p0)*10^-5)*0.001/60);
    md.calc.ChokeOpening = g_cInv(u);
    % Step simulator
    md.calc.Step();
    % Read pressure measurements
    pbReal = md.calc.CalculatedDownholePressure;
    pcReal = md.calc.CalculatedChokePressure;
    % Set global variables for observer
    z_c = md.calc.ChokeOpening;
    q_bpp = md.calc.ChokePumpFlowRateIn;
    v_d = md.calc.TopOfStringVelocity;
    % Run observer
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
    YExtended(:,n) = y;
    

    % Log variables
    qbpp(n) = md.calc.ChokePumpFlowRateIn;
    StandpipePressure(n)=md.calc.PressureChokePump;
    TopOfStringVelocity(n) = md.calc.TopOfStringVelocity;
    FlowrateIn(n)=md.calc.FlowRateIn;
    AnnulusChokeOpening(n)=md.calc.ChokeOpening;
    ChokeLineChokeOpening(n)=md.calc.ChokeLineChokeOpening;
    DensityIn(n)=md.calc.DensityIn;
    DensityOut(n)=md.calc.DensityOut;
    DensityAnnulus(n)=md.calc.DensityAnnulus;
    AnnulusPressure(n)=md.calc.CalculatedChokePressure;
    ChokeLinePressure(n)=md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Pressure');
    ChokeLinePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E2880','Measure Point Pressure');
    StandpipePressure(n)=md.calc.CalculatedSPP;
    BottomholePressure(n)=md.calc.CalculatedDownholePressure;
    BottomholePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E1','Measure Point Pressure');
    AnnulusFlowrateOut(n)=md.calc.CalculatedFlowRateOut;
    ChokeLineFlowrateOut(n)=-md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Total Flowrate');
    
    


    t = t + dT;	% Update time stamp for next step
    % Pause remainder of sample time
    t_cpu = toc; % Computation time
%     pause(dT - t_cpu);
    if mod(n,100) == 0
       n/Tstop 
    end
    if n>Tstop
        F_run = false;
    end
    n = n+1;

end

%% Do some plotting
t = 0:dT:Tstop*dT;
subplot(2,2,1);
% plot(t,pbArr,'k');
plot(t,BottomholePressure*10^-5,'k');
hold on
plot(t,262*ones(size(t)),'k--');
xlabel('time [s]');
ylabel('p_b_i_t [bar]');
legend('p_b_i_t','p_r_e_f')
ylim([255 270])
subplot(2,2,2);
plot(t,AnnulusPressure*10^-5,'k');
hold on
xlabel('time [s]');
ylabel('p_c [bar]');
ylim([1 25])
subplot(2,2,3);
plot(t,AnnulusChokeOpening,'k');
xlabel('time [s]');
ylabel('u_c [0-1]');
ylim([0.2 0.6])
subplot(2,2,4);
plot(t,cos(2*pi*(t+300*dT)/12),'k');
xlabel('time [s]');
ylabel('x_d [m]');


%% Retrieving some data from figures
% fighandle = hgload('PaperPressures5Order.fig');
% h = findobj(fighandle,'Type','line');
% x1 = get(h(500),'xdata');
% y1 = get(h(500),'ydata');
% figure
% plot(x1,y1*10^-5,'LineWidth',2);
% x2 = get(h(10),'xdata');
% y2 = get(h(10),'ydata');

% hold on
% plot(x2,y2*10^-5,'r','LineWidth',2)
% x3 = get(h(11),'xdata');
% y3 = get(h(11),'ydata');
% plot(x3,y3*10^-5,'m','LineWidth',2)
% x4 = get(h(12),'xdata');
% y4 = get(h(12),'ydata');
% plot(x4,y4*10^-5,'k','LineWidth',2)
% xlabel('time [s]');
% ylabel('p_c [bar]');
% legend('N=5','N=15','N=50','Measured')
hfig = get(gcf);
hAxes = hfig.Children;
hProperties = get(hAxes(2));
hLine = hProperties.Children;
x1 = get(hLine(1),'XData');
y1 = get(hLine(1),'YData');
x2 = get(hLine(2),'XData');
y2 = get(hLine(2),'YData');
% x3 = get(hLine(3),'XData');
% y3 = get(hLine(3),'YData');
% x4 = get(hLine(4),'XData');
% y4 = get(hLine(4),'YData');
figure
% subplot(2,2,1)
plot(x1,y1*10^-5,'r')
hold on
plot(x2,y2*10^-5)
% plot(x4,y4,'LineWidth',2)
% plot(x3,y3,'k','LineWidth',2)
% % i = 0:0.05:100;
% % plot(i,262*ones(size(i)),'k--')
% xlabel('time [s]')
% ylabel('p_b_i_t [bar]')
% legend('IRISDrill','Fitted model')
% % legend('p_b_i_t','p_r_e_f')
% 
% ylim([240 300])
% xlim([0 75])
% hProperties = get(hAxes(4));
% hLine = hProperties.Children;
% x2 = get(hLine(1),'XData');
% y2 = get(hLine(1),'YData');
% x1 = get(hLine(2),'XData');
% y1 = get(hLine(2),'YData');
% subplot(2,2,2)
% plot(x2,y2*10^-5,'r')
% hold on
% plot(x1,y1*10^-5,'k')
% legend('IRISDrill','Fitted model')
% 
% 
% xlabel('time [s]')
% ylabel('p_c [bar]')
% ylim([0 40])
% xlim([0 75])
% hProperties = get(hAxes(2));
% hLine = hProperties.Children;
% x3 = get(hLine(1),'XData');
% y3 = get(hLine(1),'YData');
% subplot(2,2,3);
% plot(x3,y3,'k')
% xlabel('time [s]')
% ylabel('u_c [0-1]')
% xlim([0 75])
% 
% hProperties = get(hAxes(1));
% hLine = hProperties.Children;
% x3 = get(hLine(1),'XData');
% y3 = get(hLine(1),'YData');
% subplot(2,2,4);
% plot(x3,y3,'k')
% xlabel('time [s]')
% ylabel('x_d [m]')
% xlim([0 75])


% i = 1:0.05:125;
% plot(i,262*ones(size(i)),'r')
% xlabel('time [s]');
% ylabel('p_1 [bar]');
% legend('Output Regulation','Internal Model','p_r_e_f','Uncontrolled case')
% ylim([255 275]);



