%%This file runs the heave compensation case on the IRISDrill simulator
%%using the output regulation controller for bottom hole pressure control.
%%Written in 2011 by Ingar Landet
clc
clear all;
global z_c  q_bpp v_d geometry xDot pbReal pcReal;

%% Initialize variables
Tstop= 2500;
FlowrateIn=zeros(Tstop,1);
AnnulusChokeOpening=zeros(Tstop,1);
ChokeLineChokeOpening=zeros(Tstop,1);
AnnulusPressure=zeros(Tstop,1);
ChokeLinePressure=zeros(Tstop,1);
ChokeLinePressure2 = zeros(Tstop,1);
StandpipePressure=zeros(Tstop,1);
BottomholePressure=zeros(Tstop,1);
BottomholePressure2 = zeros(Tstop,1);
AnnulusFlowrateOut=zeros(Tstop,1);
ChokeLineFlowrateOut=zeros(Tstop,1);
DensityIn=zeros(Tstop,1);
DensityOut=zeros(Tstop,1);
DensityAnnulus=zeros(Tstop,1);
ChokePressure= zeros(Tstop,1);
TopOfStringVelocity=zeros(Tstop,1);
qbpp = zeros(Tstop,1);
dT=0.05;
t=1.00;
F_run=true;
counter=1.00;

%% Initialize drillingcalculator
md=IRISDrill();
md.loadWellbore('U2262bar.binaryWellbore'); %Ullrigg well U2 data
md.initialize();
md.calc.StepDuration = dT;
md.calc.ChokeOpening = 1;
md.calc.ChokeLineChokeOpening = 1;
md.calc.FlowRateIn=0/60000;  % u1
md.calc.ChokePumpFlowRateIn=3000/60000; % u3 Flowrate for Choke Line Pump 
md.calc.DensityIn=1475; % u4
md.calc.ChokePumpDensityIn=1475; % u6 - choke pump fluid density  [kg/m3]
md.calc.Step();
md.calc.ChokeOpening = 0.3; % Control Choke, used as control input
md.calc.ChokeLineChokeOpening = 10^-4;  % Kill line choke, not used
t_cpu=0;
n = 1;


%% Initialize observer
% Parameters
N=5;
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1900;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1702;
geometry = findAreasAndVolumes(P);
YExtended = zeros(2*N-1, Tstop);
z_c = md.calc.ChokeOpening;
q_bpp = md.calc.ChokePumpFlowRateIn;
v_d = 0;
p_a0 = zeros(N,1);
p_c = 10^5;
p_a0(1) = p_c + 1475*9.81*P.h_dh;
for i = 2:N
   p_a0(i) = p_a0(i-1) - 1475*9.81*P.h_dh/(N-1); 
end
q_a0 = zeros(N-1,1);
y = [p_a0;q_a0];
xDot = zeros(2*N-1,1);            % allocate memory once, for speed
p_ref = 262*10^5;

%% Parameters for controller
param.beta = 30000*10^5;
param.A = sum(geometry.A_a)/N;
param.l = geometry.l_d(1);
param.k_c = 2900;
param.rho = 1475;
param.g = 9.81;
param.dh = P.h_dh;
param.d_a = 0.2304;
param.do = 0.1372;
param.omega = 2*pi/12;



%% Run to stabilize pressure
md.calc.StepDuration = 1;
for i = 1:300
    % Step simulator
    md.calc.Step();
    % Read pressure measurements
    pbReal = md.calc.CalculatedDownholePressure;
    pcReal = md.calc.CalculatedChokePressure;
    % Set global variables for observer
    z_c = md.calc.ChokeOpening;
    q_bpp = md.calc.ChokePumpFlowRateIn;
    v_d = 0;
    % Run observer
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
end
md.calc.StepDuration = dT;
while F_run
    tic	% Start timer
    
    % Move drill string
    md.calc.TopOfStringVelocity = -2*pi/12*sin(2*pi*(n+i)*dT/12);
    x = cos(2*pi*(n+i)*dT/12);
    v = md.calc.TopOfStringVelocity;
    % Get control signal
    md.calc.ChokeOpening = outputRegulationHO(x,v,p_ref,pcReal,y,param,md.calc.ChokePumpFlowRateIn,y(1));

    % Step simulator
    md.calc.Step();
    % Read pressure measurements and set global variables for observer
    pbReal = md.calc.CalculatedDownholePressure;
    pcReal = md.calc.CalculatedChokePressure;
    z_c = md.calc.ChokeOpening;
    q_bpp = md.calc.ChokePumpFlowRateIn;
    v_d = md.calc.TopOfStringVelocity;

    % Run observer
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
    YExtended(:,n) = y;
    
    % Log variables
    qbpp(n) = md.calc.ChokePumpFlowRateIn;
    StandpipePressure(n)=md.calc.PressureChokePump;
    TopOfStringVelocity(n) = md.calc.TopOfStringVelocity;
    FlowrateIn(n)=md.calc.FlowRateIn;
    AnnulusChokeOpening(n)=md.calc.ChokeOpening;
    ChokeLineChokeOpening(n)=md.calc.ChokeLineChokeOpening;
    DensityIn(n)=md.calc.DensityIn;
    DensityOut(n)=md.calc.DensityOut;
    DensityAnnulus(n)=md.calc.DensityAnnulus;
    AnnulusPressure(n)=md.calc.CalculatedChokePressure;
    ChokeLinePressure(n)=md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Pressure');
    ChokeLinePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E2880','Measure Point Pressure');
    StandpipePressure(n)=md.calc.CalculatedSPP;
    BottomholePressure(n)=md.calc.CalculatedDownholePressure;
    BottomholePressure2(n)=md.calc.GetSensorValue('DrillstringSensor_E1','Measure Point Pressure');
    AnnulusFlowrateOut(n)=md.calc.CalculatedFlowRateOut;
    ChokeLineFlowrateOut(n)=-md.calc.GetSensorValue('ChokeLineSensor_1','Measure Point Total Flowrate');
    
    


    t = t + dT;	% Update time stamp for next step
    % Pause remainder of sample time
    t_cpu = toc; % Computation time
%     pause(dT - t_cpu);
    if mod(n,100) == 0
       n/Tstop 
    end
    if n>Tstop
        F_run = false;
    end
    n = n+1;

end

%% Do some plotting
figure;
plot(BottomholePressure);
hold on
plot(YExtended(1,:),'r');
figure;
plot(AnnulusPressure);
hold on
plot(YExtended(5,:))

%%
figure
t = 0:dT:Tstop*dT;
subplot(2,2,1);
% plot(t,pbArr,'k');
plot(t,BottomholePressure,'k')
hold on
plot(t,262*10^5*ones(size(t)),'k--');
legend('p_b_i_t','p_r_e_f')
xlabel('time [s]');
ylabel('p_b_i_t [Pa]');
subplot(2,2,2);
plot(t,AnnulusPressure,'k');
hold on
xlabel('time [s]');
ylabel('p_c [Pa]');
subplot(2,2,3);
plot(t,AnnulusChokeOpening,'k');
xlabel('time [s]');
ylabel('u_c [0-1]');
subplot(2,2,4);
plot(t,cos(2*pi*(t+300*dT)/12),'k');
xlabel('time [s]');
ylabel('x_d [m]');




