clc
% clear all
load('IRISChokeSineHeave.mat')
global p_cReal p_bReal w1Real omega P z_c xDot q_p q_bpp v_d geometry geometryObs  q_c;


dT    = 0.05;                     % [s]       

N = 5;                            %  # cells in both drill string and annulus change here and in the model file to chenge the number of cells

v_d = 0;
q0 = 0*0.001/60;

%% Parameters
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1900;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1702;
geometry = findAreasAndVolumes(P);
%% Initial conditions
p_a0 = zeros(N,1);
p_c = 1.02*10^5;
p_a0(1) = p_c + 1420*9.81*P.h_dh;
for i = 2:N
   p_a0(i) = p_a0(i-1) - 1420*9.81*P.h_dh/(N-1); 
end
p_ref = p_a0(1)+9*10^5;
q_a0 = zeros(N-1,1);
x0 = [p_a0;q_a0];
z_c = 0.3;
xDot = zeros(2*N-1,1);            % allocate memory once, for speed


simPoints = length(BottomholePressure);
YExtended = zeros(2*N-1, simPoints);
q_choke = zeros(1,simPoints);
q_heave = zeros(1,simPoints);
y = x0';
q_p = 0;
q_c = q0;

param.b0 = 1/8;
param.beta = 14000*10^5;
param.A = sum(geometry.A_a)/N;
param.l = geometry.l_d(1);
param.k_c = 3200;
param.k_i = 0;
param.k_p = 0.5;
param.dT = dT;
param.timeStep = dT;
param.v_d = 0;
param.x_d = 0;
param.Lp = 1;
param.Li = 0.1;
%% Extra for feedforward controller:
param.mju = 0.1257;
param.a = 0.49;
param.b = 0.99;
param.L = P.L;
param.rho = 1420;
param.g = 9.81;
param.dh = P.h_dh;
param.v_dDot = 0;
param.v_dDotDot = 0;
param.d_a = 0.2304;
param.do = 0.1372;
param.l1 = 1.5;
param.l2 = 1.5;
param.l3 = 1.5;
param.omega = 2*pi/12;


% q_bpp = param.k_c*0.5*sqrt((10^6-10^5)*10^-5)*0.001/60
q_bpp = 3000/(1000*60);

controlZC = zeros(1, simPoints);
controlPump = zeros(1, simPoints);
pcArr = zeros(1,simPoints);
pbArr = zeros(1,simPoints);
x_dArr = zeros(1,simPoints);
v_dArr = zeros(1,simPoints);



Z_c = (1/100)*[0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
%g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );

p0 = 10^5;
w1Real = 0;
Amp = 5*10^5;
omega = 2*pi/12;
K_c = 2900;
beta = 17000*10^5;
l = 400;
A = 0.2;
beta1 = beta/(l*A);


for i = 1:simPoints
    omega = param.omega;

    
    if( i > 0)
        k = ceil((i-1000)*dT);
%         z_c = 0.3 + 0.2*sin(omega*(i+300)*dT);
            z_c = AnnulusChokeOpening(i);

            v_d = TopOfStringVelocity(i);
%         x_d = cos(omega*(i+300)*dT);
%         v_d = -omega*sin(omega*(i+300)*dT);
%         w1Real = x_d;

        if mod(i,1) == 0
%             [z_c q_bpp] = outputRegulationHO(x_d,v_d,p_ref,y(N),y,param,q_bpp);
        end

        
    end
    %% Log
    q_choke(i) = q_c;
    q_heave(i) = v_d*pi*param.do^2/4;
    controlZC(i) = z_c;
    controlPump(i) = q_bpp;
%     x_dArr(i) = x_d;
%     v_dArr(i) = v_d;

    
    % Run "real system"
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
    YExtended(:,i) = y;
    if (mod(i,100) == 0)
        
        i/simPoints
    end
    
    
end
%%
t = dT:dT:simPoints*dT;
subplot(2,2,1);
% plot(t,pbArr,'k');
plot(t,YExtended(1,:),'k');
hold on
plot(t,BottomholePressure,'r')
legend('Fitted model','IRISDrill')
xlabel('time [s]');
ylabel('p_b_i_t [Pa]');
subplot(2,2,2);
plot(t,YExtended(N,:),'k');
hold on
plot(t,AnnulusPressure,'r');
legend('Fitted model','IRISDrill')

xlabel('time [s]');
ylabel('p_c [Pa]');
subplot(2,2,3);
plot(t,controlZC,'k');
xlabel('time [s]');
ylabel('u_c [0-1]');
subplot(2,2,4);
plot(t,cos(2*pi*(t+300*dT)/12),'k');
xlabel('time [s]');
ylabel('x_d [m]');

%% performance measures
meanPressure = mean(YExtended(1,:));
peakPressure = max(abs(YExtended(1,:)-meanPressure));
peakDist = max(abs(x_dArr));
peakOverPeak = peakPressure/peakDist
RMSPressure = sqrt(mean((YExtended(1,:)-meanPressure).^2));
RMSdist = sqrt(mean(x_dArr.^2));
RMS = RMSPressure/RMSdist