function [Ydot] = fTogetherState(t,y) % Nested function for the ODE

g     = 9.81;                     % [m/s2]    Gravitational acceleration
K_c   = 2900;                      % [l/min/sqrt(bar)] Choke flow constant
p_0   = 1.0131*10^5;                        % [Pa]     Pressure outside the system

global z_c  q_bpp v_d geometry q_c xDot pbReal pcReal;

% Parameter added from the PDE block
N = 5;                            %  # cells in both drill string and annulus
l = geometry.l_d(1);
dh = 1702/(N-1);
do = 0.1372; 
A = sum(geometry.A_a)/N;
d_a = 0.2304;
mju = 0.5;
beta = 30000*10^5;
a_an = 0.49;
b_an = 0.99;
rho_old = 1475;
Ao = pi*do^2/4;

p2a = 5.0909e+04;
k1 = (1/(N-0.5))*p2a;
k2 = 32*mju*(a_an+b_an)*l/(A*(d_a^2-do^2));




Z_c = (1/100)*[0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );


p_aDot = xDot(1:N);
q_aDot = xDot(N+1:2*N-1);

p_a = y(1:N);
q_a = y(N+1:2*N-1);



rho = rho_old;

p_c = p_a(N);

if p_c < p_0
    p_c = p_0;
    q_out = 0;
else
    q_out = K_c*g_c(z_c)*sqrt(1400/rho)*sqrt((p_c-p_0)*10^-5)*0.001/60;    %flow through choke
end
q_c = q_out;


p_aDot(1) = beta/(A*l)*(-q_a(1)-0.9*v_d*Ao);% - 20*(p_a(1)-pbReal);
p_aDot(N) = beta/(A*l)*(q_a(N-1)-q_out+q_bpp) - 20*(p_a(N)-pcReal);

for k = 2:N-1
    p_aDot(k) = beta/(A*l)*(q_a(k-1)-q_a(k));
end

for k = 1:N-1
    if k == 1
        fac = 0.3;
        q = q_a(k);
        q_aDot(k) = (A*fac/(l*rho))*(p_a(k)-p_a(k+1))-(k1+(1/fac)*k2*q)*(fac*A/(l*rho)) - g*A*fac*dh/l;

    else
        q = q_a(k);
        q_aDot(k) = (A/(l*rho))*(p_a(k)-p_a(k+1))-(k1+k2*q)*(A/(l*rho)) - g*A*dh/l;
    end
end


Ydot = [p_aDot; q_aDot];



end