function [z_c,q_bpp] = outputRegulationHO(x_d,v_d,p_ref,pc,y,param,q_bpp,pb)
%Implements the output regulation controller for a 5-volum version of the
%hydraulic model. 
%Written by Ingar Landet, 2011

% Read parameters for controller
beta = param.beta;
A = param.A;
l = param.l;
k_c = param.k_c;
p0 = 1.0131*10^5;
do = param.do;
d_a = param.d_a;
rho = param.rho;
g = param.g;
dh = param.dh/4;
omega = param.omega;
mju = 0.5;
a_an = 0.49;
b_an = 0.99;
p2a = 5.0909e+04;
k1 = 1.3;
k2 = 0.3;
Ao = pi*do^2/4;
beta1 = beta/(l*A);
beta2 = A/(l*rho);
beta4 = rho*g*dh;
kf1 = (1/4.5)*p2a;
kf2 = 32*mju*(a_an+b_an)*l/(A*(d_a^2-do^2));

%Initialize controller variables
p = [0;0;0;0;0;0;0;0;0];
x = [pc;y(9);y(4);y(8);y(3);y(7);y(2);y(6);pb];

% Solve the regulator equations
p(9) = p_ref;
p(8) =  -k1*v_d*Ao;
p8D = k1*omega^2*x_d*Ao;
p(7) = p(9) - p8D/(beta2*k2) - kf1 - kf2*p(8)/k2 - beta4;
p82D = k1*omega^2*v_d*Ao;
p7D = -p82D/(beta2*k2) -kf2*p8D/(k2);
p(6) = p(8) - p7D/beta1;
p83D = -k1*omega^4*x_d*Ao;
p72D = -p83D/(beta2*k2) - kf2*p82D/k2;
p6D = p8D - p72D/beta1;
p(5) = p(7) - p6D/beta2 - kf1 - kf2*p(6) - beta4;
p84D = -k1*omega^4*v_d*Ao;
p73D = -p84D/(beta2*k2) - kf2*p83D/k2;
p62D = p82D - p73D/beta1;
p5D = p7D - p62D/beta2 - kf2*p6D;
p(4) = p(6) - p5D/beta1;
p85D = k1*omega^6*x_d*Ao;
p74D = -p85D/(beta2*k2) -kf2*p84D/k2;
p63D = p83D - p74D/beta1;
p52D = p72D - p63D/beta2 - kf2*p62D;
p4D = p6D - p52D/beta1;
p(3) = p(5) - p4D/beta2 - kf1 -kf2*p(4) - beta4;
p86D = k1*omega^6*v_d*Ao;
p75D = -p86D/(beta2*k2) -kf2*p85D/k2;
p64D = p84D - p75D/beta1;
p53D = p73D - p64D/beta2 - kf2*p63D;
p42D = p62D - p53D/beta1;
p3D = p5D - p42D/beta2 -kf2*p4D;
p(2) = p(4) - p3D/beta1;
p87D = -k1*omega^8*x_d*Ao;
p76D = -p87D/(beta2*k2) -kf2*p86D/k2;
p65D = p85D - p76D/beta1;
p54D = p74D - p65D/beta2 - kf2*p64D;
p43D = p63D - p54D/beta1;
p32D = p52D - p43D/beta2 -kf2*p42D;
p2D = p4D - p32D/beta1;
p(1) = p(3) - p2D/beta2 - kf1 -kf2*p(2) - beta4;
p88D = -k1*omega^8*v_d*Ao;
p77D = -p88D/(beta2*k2) -kf2*p87D/k2;
p66D = p86D - p77D/beta1;
p55D = p75D - p66D/beta2 - kf2*p65D;
p44D = p64D - p55D/beta1;
p33D = p53D - p44D/beta2 -kf2*p43D;
p22D = p42D - p33D/beta1;
p1D = p3D - p22D/beta2 - kf2*p2D;


% Calculate the control signal
if p(1) <= p0
    u = 10^-4;
else
    c = (p(2) + q_bpp - p1D/beta1)/(k_c*sqrt(1400/rho)*sqrt((p(1)-p0)*10^-5)*0.001/60);
    K = 10^-1*[10^-6 0 0 0 0 0 0 0 10^-6];
    u = c + K*(x-p);
end


% Choke characteristics
Z_c = (1/100)*[0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );

z_c = g_cInv(u);


end