%%This file runs the Statoil controller on the Kaasa model for the case
%%tested on Ullrigg, to show that, if the model hade been sufficient, the
%%controller would have worked.
%%Written in 2011 by Ingar Landet

clc
clear all
% Global variables to communicate with ode solver. Bad programming, but
% works for the purpose.
global z_c q_bpp beta V v_d A_d k_c;


%Heave simulation
% Read from logged Ullrigg data
load('Ullrigg_Eval05_20091126_1328.mat');
interval = 960:1200;
h_b = LOG.h_blkRig(interval);
v_EstUllRigg = LOG.v_blk(interval);
v_b = zeros(size(h_b));
p_ref = LOG.pRef_c(interval);
z_choke = LOG.z_c(interval);
q_pump = LOG.q_bpp(interval)*0.001/60;%q_pump = LOG.omega_p(interval)*16.7*0.001/60;
q_choke = LOG.q_c(interval)*0.001/60;
for i =1:length(h_b)
    if(isnan(h_b(i)))
        h_b(i) = h_b(i-1);
    end
    if(isnan(v_EstUllRigg(i)))
        v_EstUllRigg(i) = v_EstUllRigg(i-1);
    end
    if(isnan(q_choke(i)))
        q_choke(i) = q_choke(i-1);
    end
    if(isnan(z_choke(i)))
        z_choke(i) = z_choke(i-1);
    end
    if(isnan(q_pump(i)))
        q_pump(i) = q_pump(i-1);
    end
    if(isnan(p_ref(i)))
        p_ref(i) = p_ref(i-1);
    end
    if i > 1
        v_b(i) = (h_b(i) - h_b(i-1));
    end
end

[B A] = butter(2,0.4,'low');
v_b = filter(B,A,v_b);
v_b = [v_b(1:4); v_b(8:end); v_b(end-4:end)];

% Initialize
dT = 0.05;
simPoints = 50 + length(q_pump)/dT;
p0 = 10^5;
beta = 17200*10^5;
V = 54;
A_d = 0.0098;
k_c = 538;
Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
p_c = p_ref(1)*10^5;
y = p_c;
p_r = p_ref(1)*10^5;
p_hat = 41*10^5;
q_err = 0.01;
dT = 0.05;
omega = pi/6;
z_c = 35;
YExtended = zeros(3,simPoints);
param.beta = beta;
param.A_d = A_d;
param.V_a = V;
param.k_c = k_c;
param.dT = dT;
param.v_d = 0;
param.Lp = 1;
param.Li = 0.1;
param.k_p = 0.2; 
controlZC = zeros(1,simPoints);
phatArr = zeros(1,simPoints);
qerrArr = zeros(1,simPoints);
q_bpp = 0.35*k_c*sqrt((p_ref-p0)*10^-5)*0.001/60;
v_d = 0;

for i=1:simPoints
    z_c = z_choke(1);
    q_bpp = q_pump(1);
    if i>50
        %     v_d = -omega*sin(omega*i*dT);
        k = ceil((i-50)*dT);
        v_d = v_b(k);
        q_bpp = q_pump(k);
        param.v_d = v_d;
        q_c = k_c*g_c(z_c)*sqrt(1400/1420)*sqrt((p_c-p0)*10^-5)*0.001/60;
        % Calculate control signal and observer values
        [z_c p_hat q_err] = statoilController(q_c,q_bpp,p_c,param,p_r,p_hat,q_err);
    end
    % Log variables
    controlZC(i) = z_c;
    phatArr(i) = p_hat;
    qerrArr(i) = q_err;
    % Run ode solver
    y = ode4(@Kaasa,[0 dT],y);
    y = y(end,:);
    p_c = y(1);

    YExtended(:,i) = y;
    
    if mod(i,100) == 1
        i/simPoints
    end
end
%% Plot Figure
t = dT:dT:simPoints*dT;
subplot(2,2,1);
plot(t,YExtended(1,:),'k');
xlabel('time (s)');
ylabel('p_c (Pa)');
subplot(2,2,2);
plot(t,YExtended(1,:),'k');
xlabel('time (s)');
ylabel('p_b_i_t (Pa)');
subplot(2,2,3);
plot(t,controlZC,'k');
xlabel('time');
ylabel('u_c (%)');
subplot(2,2,4)
plot(h_b,'k');
xlabel('time (s)');
ylabel('x_d (m)')
