function [z_c p_hat qerr] = statoilController(q_c,qbpp,p_c,param,p_ref,p_hat, qerr)
%Implements the feedback linearizing controller with observer used in
%Ullrigg tests

beta = param.beta;
V_a = param.V_a;
k_c = param.k_c;
p0 = 10^5;
dT = param.dT;
A_d = param.A_d;
v_d = param.v_d;
Lp = param.Lp;
Li = param.Li;
k_p = param.k_p;

Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
%g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );


%% Statoil controller:

    function yDot = obs(t,y)
        p_hat = y(1);
        qerr = y(2);
        p_hatDot = beta/(V_a)*(qbpp - A_d*v_d - q_c + qerr - Lp*(V_a/beta)*(p_hat-p_c));
        qerrDot = -Li*(V_a/beta)*(p_hat-p_c);
        yDot = [p_hatDot qerrDot]';     
    end

y0 = [p_hat qerr]';

y = ode4(@obs,[0:0.01:dT], y0);
y = y';
y = y(:,end);
p_hat = y(1);
qerr = y(2);


if p_c <= p0
    z_c = 0;
else
    q_cStar = qbpp - A_d*v_d +0 - (V_a/beta)*k_p*(p_ref-p_c);
    u = (q_cStar)/(k_c*sqrt(1400/1420)*sqrt((p_c-p0)*10^-5)*0.001/60);
    z_c = g_cInv(u);
end




end