%%This file runs the test bech for the down hole pressure regulation. It is
%%possible to change which controller to use, as well as changing the
%%disturbance model used.
%%In the end, there is also a section that can make an mpg movie, that is
%%the reason for all the "mpgwrite"-files in this folder.
%%Written in 2011 by Ingar Landet

clc
clear all
% Global variables to communicate with ode solver. Bad programming, but
% works for the purpose
global p_cReal p_bReal w1Real omega P z_c xDot q_p q_bpp v_d geometry geometryObs  q_c;


dT    = 0.05;                     % [s]       
N = 5;                            %  # cells in both drill string and annulus change here and in the model file to chenge the number of cells
v_d = 0;
q0 = 0*0.001/60;

%% Model Parameters
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1990.99;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1540;
geometry = findAreasAndVolumes(P);

%% Initial conditions
p_a0 = zeros(N,1);
p_ref = 240*10^5;
p_a0(1) = p_ref - 1420*9.81*P.h_dh/(2*N);
for i = 2:N
   p_a0(i) = p_a0(i-1) - 1420*9.81*P.h_dh/N; 
end
q_a0 = zeros(N-1,1);
x0 = [p_a0;q_a0];
z_c = 34.5;
xDot = zeros(2*N-1,1);            % allocate memory once, for speed

%% Initials for waves
e1 = 2*pi*rand(1);
e2 = 2*pi*rand(1);
e3 = 2*pi*rand(1);
w1 = 0.0667*rand(1)+0.35;
w2 = 0.0667*rand(1)+0.4167;
w3 = 0.0667*rand(1)+0.4833;

simPoints = 4000;
YExtended = zeros(2*N-1, simPoints);
q_choke = zeros(1,simPoints);
q_heave = zeros(1,simPoints);
y = x0';
q_p = 0;
q_c = q0;

%% IMC controller
Q0 = 10^-7;
Q1 = 10^-7;
s = tf('s');
w1hat = 0.0667*rand(1)+0.35;
w2hat = 0.0667*rand(1)+0.4167;
w3hat = 0.0667*rand(1)+0.4833;
% Use 3 distinct frequencies
% param.cSys = Q0 + Q0/s + (Q0/2)*s/(s^2+w1hat^2) + (Q0/2)*s/(s^2+w2hat^2) + (Q0/2)*s/(s^2+w3hat^2);
% Use 1 single frquency
param.cSys = Q0 + Q0/s + Q0*s/(s^2+(pi/6)^2);
% In case of feedback linearization, have to use a partial derivative
% action:
% param.cSys = param.cSys*0.01*s/(0.01*s+1);
% Initial conditions for the controller
xW = [0 0 0];
%Initials for controller with feedback lin and observer:
xW2 = [0 0];

%% Repetitive controller
w = 5/(s+5);
delay = pade(exp(-11.8*s),5);
sys1 = feedback(w*delay,1,+1);
sys2 = 10^-9*s^2/(0.001*s+1);
param.cSysRep = sys1*sys2;
% repCon = w*delay/(1-w*delay) + Q0;
% sys1 = feedback(ss(w*exp(-s*11.8)),1,+1)*10^-7 + Q0;
% sys2 = (s^9 + 134*s^8 + 7946*s^7 +2.736*10^5*s^6+6.031*10^6*s^5+8.819*10^7*s^4+8.557*10^8*s^3+5.311*10^9*s^2 +1.913*10^10*s + 3.047*10^10)/(590.5/(30^9)*(s+30)^9);
% sys2 = (s^3+9.109*s^2+4.778*s+6.847)/((0.4974/10^3)*(s+10)^3);
% param.cSysRep = pade(sys1,5);
xC = [0 0 0 0 0 0 0];


%% Parameters for controllers
param.b0 = 1/8;
param.beta = 14000*10^5;
param.A = sum(geometry.A_a)/N;
param.l = geometry.l_d(1)*N/(N-0.5);
param.k_c = 545;
param.k_i = 0;
param.k_p = 0.5;
param.dT = dT;
param.timeStep = dT;
param.v_d = 0;
param.x_d = 0;
param.Lp = 1;
param.Li = 0.1;
%% Extra for feedforward controller:
param.mju = 0.1257;
param.a = 0.49;
param.b = 0.99;
param.L = P.L;
param.rho = 1420;
param.g = 9.81;
param.dh = P.h_dh;
param.v_dDot = 0;
param.v_dDotDot = 0;
param.d_a = 0.2304;
param.do = 0.1372;
param.l1 = 1.5;
param.l2 = 1.5;
param.l3 = 1.5;
param.omega = 2*pi/12;
param.omegaArr = [0.0667*rand(1)+0.35 0.0667*rand(1)+0.4167 0.0667*rand(1)+0.4833];
% param.omegaArr = [w1 w2 w3];

q_bpp = param.k_c*0.4*sqrt((getPc(y,P)-10^5)*10^-5)*0.001/60;

controlZC = zeros(1, simPoints);
controlPump = zeros(1, simPoints);
pcArr = zeros(1,simPoints);
pbArr = zeros(1,simPoints);
x_dLog = zeros(1,simPoints);
v_dLog = zeros(1,simPoints);
YhatExtended = zeros(9,simPoints);

%% Initials for observer
P.N = 5;
geometryObs = findAreasAndVolumes(P);
P.N = N;
p_a0(1) = p_ref - 1420*9.81*P.h_dh/(10);
for i = 2:5
   p_a0(i) = p_a0(i-1) - 1420*9.81*P.h_dh/5; 
end
y_hat = [p_a0(1:5);zeros(4,1)];



Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
%g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );

p0 = 10^5;
w1Real = 0;

for i = 1:simPoints
    p_c1 = getPc(y,P);
    p_cReal = p_c1;
    p_b1 = getPb(y,P);
    p_bReal = p_b1;
    omega = param.omega;
    x_d = 0;
    v_d = 0;

    
    if( i > 50)
        k = ceil((i-50)*dT); 
        A = 1;
        %% Generate waves:
        % three frequencies
%         x_d = sqrt(2*2.45*0.0667/2)*cos(w1*i*dT+e1) + sqrt(2*7.35*0.0667/2)*cos(w2*i*dT+e2) + sqrt(2*1.67*0.0667/2)*cos(w3*i*dT+e3);
%         v_d = -w1*sqrt(2*2.45*0.0667/2)*sin(w1*i*dT+e1) - w2*sqrt(2*7.35*0.0667/2)*sin(w2*i*dT+e2) - w3*sqrt(2*1.67*0.0667/2)*sin(w3*i*dT+e3);
%         xdArr = [sqrt(2*2.45*0.0667/2)*cos(w1*i*dT+e1) sqrt(2*7.35*0.0667/2)*cos(w2*i*dT+e2) sqrt(2*1.67*0.0667/2)*cos(w3*i*dT+e3)]';
%         vdArr = [-w1*sqrt(2*2.45*0.0667/2)*sin(w1*i*dT+e1),
%         -w2*sqrt(2*7.35*0.0667/2)*sin(w2*i*dT+e2), -w3*sqrt(2*1.67*0.0667/2)*sin(w3*i*dT+e3)]';

        %One frequency
        x_d = cos(omega*i*dT);
        v_d = -omega*sin(omega*i*dT);
        w1Real = x_d;

        % Calculate control signal
        if mod(i,1) == 0
%             [z_c q_bpp] = outputRegulationHO3harmonic(xdArr,vdArr,p_ref,p_c1,y_hat,param,q_bpp);
%             [z_c q_bpp] = outputRegulationHO(x_d,v_d,p_ref,p_c1,y_hat,param,q_bpp);
%             [z_c xW q_bpp] = IMC(p_c1,p_b1,param,xW);
%             [z_c xC q_bpp] = repetitive(p_c1,p_b1,y(ceil(2*N-N/2)),param,xC);
%             [z_c xW2 q_bpp] = IMCObserver(y_hat,p_c1,param,xW2);
        end

        
    end
    %% Log variables
    q_choke(i) = q_c;
    q_heave(i) = v_d*pi*param.do^2/4;
    controlZC(i) = z_c;
    controlPump(i) = q_bpp;
    pcArr(i) = p_c1;
    pbArr(i) = p_b1;
    x_dLog(i) = x_d;
    v_dLog(i) = v_d;
    
    %% Run observer
    y_hat = ode4(@fObserver,[0 dT],y_hat);
    y_hat = y_hat';
    y_hat = y_hat(:,end);
    YhatExtended(:,i) = y_hat;
    
    %% Run "real system"
    y = ode4(@fTogetherState,[0 dT],y);
    y = y';
    y = y(:,end);
    YExtended(:,i) = y;
    if (mod(i,100) == 0)
        
        i/simPoints
    end
    
    
end
%% Plot Figure
t = dT:dT:simPoints*dT;
subplot(2,2,1);
% plot(t,pbArr,'k');
plot(t,pbArr(1,:),'k');
xlabel('time (s)');
ylabel('p_b_i_t (Pa)');
subplot(2,2,2);
plot(t,pcArr,'k');
xlabel('time (s)');
ylabel('p_c (Pa)');
subplot(2,2,3);
plot(t,controlZC,'k');
xlabel('time (s)');
ylabel('u_c (%)');
subplot(2,2,4);
plot(t,x_dLog,'k');
xlabel('time (s)');
ylabel('x_d (m)');

%% performance measures
meanPressure = mean(YExtended(1,:));
peakPressure = max(abs(YExtended(1,:)-meanPressure));
peakDist = max(abs(x_dLog));
peakOverPeak = peakPressure/peakDist
RMSPressure = sqrt(mean((YExtended(1,:)-meanPressure).^2));
RMSdist = sqrt(mean(x_dLog.^2));
RMS = RMSPressure/RMSdist


%%
%% Make movie
% clc
% fig1 = figure(102);
% clf;
% winsize = get(fig1,'Position');
% winsize(1:2) = [0 0];
% numframes = simPoints/10;
% A = moviein(numframes,fig1,winsize);
% XLIM = [1 5];
% YLIM = [1*10^5 230*10^5];
% % Pressure profile
% subplot(2,2,1:2), hold on;
% % YLIM=[200 350];
% T=title('Heave Compensation');
% set(T,'FontSize',16)
% % L_pdh=line(t(1),p_dh(1),'linewidth',1.0,'color','k','linestyle','-',...
% %     'marker','x','markersize',4);
% L_pdhEst=line([1:5],YExtended(1:5,1),'linewidth',1.5,'color','b','linestyle','-');
% % line(t(1),pHat_dh_dhe05(1),'linewidth',2.0,'color','k','linestyle','-'),
% xlim(XLIM)
% ylim(YLIM)
% grid on
% xlabel('Control Volumes. #1=bit pressure, #5=choke pressure')
% ylabel('Pressure  {\itp} [Pa]')
% % text(0.7,-2,'Bit pressure')
% % legend('{\itp_{dh}} measured',...
% %     '{\itp_{dh}} estimate (high gain)',...
% % 	'{\itp_{dh}} estimate (low gain)',...
% %     'location','northeast')
% legend boxoff
% 
% 
% % Drill string pos
% subplot(2,2,3), hold on;
% % YLIM=[0.8 1.5];
% %
% L_x_d=line(0,x_dLog(1),'linestyle','-','color','b','linewidth',1.0);
% % line(t,thetaHat1_dhe05,'linestyle','-','linewidth',2.0,'color','k'), 
% % L_thetaRho=line(t(1),thetaHat2(1),'linestyle','-.','color','b','linewidth',1.0);
% % line(t,thetaHat2_dhe05,'linewidth',2.0,'color','k','linestyle','-.'), 
% xlim([0 100]);
% ylim([-1.2 1.2]);
% grid on
% xlabel('Time  {\itt} [s]')
% ylabel('Drill string position  [m]')
% % legend('{\it\theta_{F}}  (high gain)',...
% %     '{\it\theta_{F}}  (low gain)',...
% %     '{\it\theta_{\rho}}  (high gain)',...
% %     '{\it\theta_{\rho}}  (low gain)',...
% %     'location','northeast')
% legend boxoff
% 
% % Choke opening
% subplot(2,2,4), hold on;
% % YLIM=[0.8 1.5];
% %
% L_u_c=line(0,controlZC(1),'linestyle','-','color','b','linewidth',1.0);
% % line(t,thetaHat1_dhe05,'linestyle','-','linewidth',2.0,'color','k'), 
% % L_thetaRho=line(t(1),thetaHat2(1),'linestyle','-.','color','b','linewidth',1.0);
% % line(t,thetaHat2_dhe05,'linewidth',2.0,'color','k','linestyle','-.'), 
% xlim([0 100]);
% ylim([10 50]);
% grid on
% xlabel('Time  {\itt} [s]')
% ylabel('Choke opening  [%]')
% % legend('{\it\theta_{F}}  (high gain)',...
% %     '{\it\theta_{F}}  (low gain)',...
% %     '{\it\theta_{\rho}}  (high gain)',...
% %     '{\it\theta_{\rho}}  (low gain)',...
% %     'location','northeast')
% legend boxoff
% 
% % Create movie frames
% N = numframes;
% % Add the current time
% TXT = text(18.1,1.7,['t = ' num2str( 10 ) ' sec']);
% set(TXT,'FontSize',18,'FontWeight','bold')
% for k=1:N
% 	set(L_pdhEst,'XData',[1:5],'YData',YExtended(1:5,k*5))
% 	set(L_x_d,'XData',[1:k*5]./20,'YData',x_dLog([1:k*5]))    
% 	set(L_u_c,'XData',[1:k*5]./20,'YData',controlZC([1:k*5]))    
%     % Add time counter
%     set(TXT,'String',['t = ' num2str( round(k*5/20) ) ' sec'])
%     % Create movie frames
%     A(:,k)=getframe(gcf); % Create movie matrix
% end
% 
% %% Create mpeg file
% mpgwrite(A,jet,'movie.mpg');


