function [Ydot] = fTogetherState(t,y) 
% Implements the "real" system

g     = 9.81;                     % [m/s2]    Gravitational acceleration
K_c   = 545;                      % [l/min/sqrt(bar)] Choke flow constant
p_0   = 10^5;                        % [Pa]     Pressure outside the system

global z_c  q_bpp v_d geometry q_c xDot;

% Parameter added from the PDE block
N = 5;                            %  # cells in both drill string and annulus
l = geometry.l_d(1)*N/(N-0.5);
dh = 1540/N;
do = 0.1372; sum(geometry.D_dO)/N;
A = sum(geometry.A_a)/N;
d_a = 0.2304; %sqrt(4*A_a/pi+do^2);
% di = 0.1195;% sum(geometry.D_dI)/N;
% do = 0.1372; % sum(geometry.D_dO)/N;
% A_d = 0.0098; %sum(geometry.A_d)/N;
% A_a = 0.0269; %sum(geometry.A_a)/N;
% d_a = 0.2304; %sqrt(4*A_a/pi+do^2);
mju = 0.1257;%0.4207;%0.3657;
beta = 14000*10^5;
a_an = 0.49;
b_an = 0.99;
rho_old = 1420;
Ao = pi*do^2/4;

p2a = 5.0909e+04;
k1 = (1/N-0.5)*p2a;
k2 = 32*mju*(a_an+b_an)*l/(A*(d_a^2-do^2));




Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );


p_aDot = xDot(1:N);
q_aDot = xDot(N+1:2*N-1);

p_a = y(1:N);
q_a = y(N+1:2*N-1);



rho = rho_old;

p_c = p_a(N) - rho*g*dh/2 - (k1+k2*q_a(N-1))/2;

if p_c < p_0
    p_c = p_0;
    q_out = 0;
else
    q_out = K_c*g_c(z_c)*sqrt(1400/rho)*sqrt((p_c-p_0)*10^-5)*0.001/60;    %flow through choke
end
q_c = q_out;


p_aDot(1) = beta/(A*l)*(-q_a(1)-v_d*Ao);
p_aDot(N) = beta/(A*l)*(q_a(N-1)-q_out + q_bpp);

for k = 2:N-1
    p_aDot(k) = beta/(A*l)*(q_a(k-1)-q_a(k));
end

for k = 1:N-1
    q = q_a(k);
    q_aDot(k) = (A/(l*rho))*(p_a(k)-p_a(k+1))-(k1+k2*q)*(A/(l*rho)) - g*A*dh/l;
end


Ydot = [p_aDot; q_aDot];



end