function [z_c,q_bpp] = outputRegulationHO(x_d,v_d,p_ref,p_c,y,param,q_bpp)
% Implements the output regulation controller for a 5-volume version of the
% hydraulic model and a single harmonic disturbance.


b0 = param.b0;
beta = param.beta;
A = param.A;
l = param.l;
k_c = param.k_c;
p0 = 10^5;
k_i = param.k_i;
dT = param.dT;
L = param.L;
do = param.do;
d_a = param.d_a;
rho = param.rho;
g = param.g;
dh = param.dh/5;
omega = param.omega;
p_cRef = p_ref - rho*g*5*dh;

p_ref = p_ref - rho*g*dh/2;

mju = 0.1257;%0.4207;%0.3657;
a_an = 0.49;
b_an = 0.99;
Re_crit = 1654;
y1 = 16/Re_crit;
y2 = -1/(16*log10(5.74/15000^0.9));
a = (y2-y1)/(15000-Re_crit);
b = y1 - a*Re_crit;
p2a = 5.0909e+04;


q_bpp = k_c*0.35*sqrt(1400/1420)*sqrt((p_cRef-p0)*10^-5)*0.001/60;%1000*0.001/60;
Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
%g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );
g_cInv=@(g) interp1( G_c, Z_c, max(0, min(g, 1)) );



Ao = pi*do^2/4;
p = [0;0;0;0;0;0;0;0;0];
x = [y(5);y(9);y(4);y(8);y(3);y(7);y(2);y(6);y(1)];

beta1 = beta/(l*A);
beta2 = A/(l*rho);
beta4 = rho*g*dh;
k1 = (1/4.5)*p2a;
k2 = 32*mju*(a_an+b_an)*l/(A*(d_a^2-do^2));

p(9) = p_ref;
p(8) =  -v_d*Ao;
p8D = omega^2*x_d*Ao;
p(7) = p(9) - p8D/beta2 - k1 - k2*p(8) - beta4;
p82D = omega^2*v_d*Ao;
p7D = -p82D/beta2 -k2*p8D;
p(6) = p(8) - p7D/beta1;
p83D = -omega^4*x_d*Ao;
p72D = -p83D/beta2 - k2*p82D;
p6D = p8D - p72D/beta1;
p(5) = p(7) - p6D/beta2 - k1 - k2*p(6) - beta4;
p84D = -omega^4*v_d*Ao;
p73D = -p84D/beta2 - k2*p83D;
p62D = p82D - p73D/beta1;
p5D = p7D - p62D/beta2 - k2*p6D;
p(4) = p(6) - p5D/beta1;
p85D = omega^6*x_d*Ao;
p74D = -p85D/beta2 -k2*p84D;
p63D = p83D - p74D/beta1;
p52D = p72D - p63D/beta2 - k2*p62D;
p4D = p6D - p52D/beta1;
p(3) = p(5) - p4D/beta2 - k1 -k2*p(4) - beta4;
p86D = omega^6*v_d*Ao;
p75D = -p86D/beta2 -k2*p85D;
p64D = p84D - p75D/beta1;
p53D = p73D - p64D/beta2 - k2*p63D;
p42D = p62D - p53D/beta1;
p3D = p5D - p42D/beta2 -k2*p4D;
p(2) = p(4) - p3D/beta1;
p87D = -omega^8*x_d*Ao;
p76D = -p87D/beta2 -k2*p86D;
p65D = p85D - p76D/beta1;
p54D = p74D - p65D/beta2 - k2*p64D;
p43D = p63D - p54D/beta1;
p32D = p52D - p43D/beta2 -k2*p42D;
p2D = p4D - p32D/beta1;
p(1) = p(3) - p2D/beta2 - k1 -k2*p(2) - beta4;
p88D = -omega^8*v_d*Ao;
p77D = -p88D/beta2 -k2*p87D;
p66D = p86D - p77D/beta1;
p55D = p75D - p66D/beta2 - k2*p65D;
p44D = p64D - p55D/beta1;
p33D = p53D - p44D/beta2 -k2*p43D;
p22D = p42D - p33D/beta1;
p1D = p3D - p22D/beta2 - k2*p2D;



if p(1) <= p0
    u = 0;
else
    c = (p(2) + q_bpp - p1D/beta1)/(k_c*sqrt(1400/1420)*sqrt((p_c-p0)*10^-5)*0.001/60);
    K = 10^-1*[10^-6 10 10^-6 10 10^-6 10 10^-6 20 10^-6];
    u = c + K*(x-p);
%     u = u/2;
end

z_c = g_cInv(u);


end