%%This file runs the Ullrigg data analysis simulations. It is possible to
%%change the parameters tested in the master thesis, like rate limitations,
%%friction parameters and time delays.
%%Written in 2011 by Ingar Landet

clc
clear all
% using a lot of global variables to communicate with ode solver. Bad
% programming, but works for this purpose.
global  P z_c xDot rho_in beta_in q_p p_c rho_change p1_temp p2_temp w_dr q_bpp v_d geometry STATE steadyState q_c;


dT    = 0.05;                     % [s]       

% Parameter added from the PDE block
N = 15;                            %  # cells in both drill string and annulus
DP0 = 50*10^5;



%Heave simulation, read from logged Ullrigg data
load('Ullrigg_Eval05_20091126_1328.mat');
interval = 960:1200;
h_b = LOG.h_blkRig(interval);
v_EstUllRigg = LOG.v_blk(interval);
v_b = zeros(size(h_b));
p_ref = LOG.pRef_c(interval);
z_choke = LOG.z_c(961:1201);
u_choke = LOG.u_c(interval);
q_pump = LOG.q_bpp(interval)*0.001/60;%q_pump = LOG.omega_p(interval)*16.7*0.001/60;
q_choke = LOG.q_c(interval)*0.001/60;
% Remove NaN-values because of lost samples
for i =1:length(h_b)
    if(isnan(h_b(i)))
        h_b(i) = h_b(i-1);
    end
    if(isnan(v_EstUllRigg(i)))
        v_EstUllRigg(i) = v_EstUllRigg(i-1);
    end
    if(isnan(q_choke(i)))
        q_choke(i) = q_choke(i-1);
    end
    if(isnan(z_choke(i)))
        z_choke(i) = z_choke(i-1);
    end
    if(isnan(u_choke(i)))
        u_choke(i) = u_choke(i-1);
    end
    if(isnan(q_pump(i)))
        q_pump(i) = q_pump(i-1);
    end
    if(isnan(p_ref(i)))
        p_ref(i) = p_ref(i-1);
    end
    if i > 1
        v_b(i) = (h_b(i) - h_b(i-1));
    end
end

%Filter to find drill string velocity, then time shift to eliminate lag
[B A] = butter(2,0.4,'low');
v_b = filter(B,A,v_b);
v_b = [v_b(1:4); v_b(8:end); v_b(end-4:end)];


% Initialize model
v_d = 0;
q0 = 0*0.001/60;
w_dr = 0;
Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1990.99;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
P.h_dh = 1540;
geometry = findAreasAndVolumes(P);
x0 = initSystem(q0,N,geometry,LOG.p_c(interval(1))*10^5);
x0 = [x0, 0];
z_c = z_choke(1);
rho_in = 1400;
beta_in = 14000*10^5;
rho_change = false;
p1_temp = x0(4*N+1);
p2_temp = x0(4*N+2);
xDot = zeros(4*N + 5,1);            % allocate memory once, for speed
y = x0';
q_p = 0;
q_c = q0;
q_bpp = q_pump(1);
STATE = 'flow';
steadyState = false;

%initialize logs
simPoints = 50*dT + length(q_pump);
YExtended = zeros(4*N + 5, simPoints/dT);
q_choke = zeros(1,simPoints/dT);
q_heave = zeros(1,simPoints/dT);
controlZC = z_c*ones(1,simPoints/dT);
actualZC = z_c*ones(1,simPoints/dT);


%Parameters for controller
param.V_a = LOG.V_a(100);
param.k_p = LOG.Kmb_p(100);
param.Lp = LOG.L_1(100);
param.Li = LOG.L_2(100);
param.beta = LOG.beta_a(100)*10^5;
param.k_c = 538;
param.dT = dT;
param.A_d = 0.0098;
param.v_d = v_d;
param.b0 = 15;
param.beta = 17250*10^5;
param.A = 0.0273;
param.l = 137.3097;

p_hat = 40*10^5;
qerr = 0;

%Values for data analysis, friction, rate limitation etc
rateLimit = inf;
xss = z_choke(1);
v_old = 0;
I = false;
control = z_choke(1);

for i = 1:simPoints/dT  
    if( i > 50)
        k = ceil((i-50)*dT); 
%         q_bpp = q_pump(k);
%         z_c = z_choke(k);
        v_d = v_b(k);
        param.v_d = v_d;
        % Statoil controller:
        if mod(i,1) == 0
           [control p_hat qerr] = statoilController(q_c,q_bpp,p_c,param,p_ref(k)*10^5,p_hat,qerr);
        end
        %low pass filter control signal to emulate physical signal
        controlZC(i) = (0.1/0.15)*controlZC(i-1) + (0.05)/(0.15)*control;
        %friction in choke
        [actualZC(i) v_old I xss] = getRealChokeOpening(controlZC(i),controlZC(i-1),v_old,I,xss,z_c);
        %Introduce time delay by saying z_c = actualZC(i-d*dT)
        z_c = actualZC(i);
        
        q_bpp = q_pump(k);
    end
    %Log variables
    q_choke(i) = q_c;
    q_d = y(N+1:2*N-1);
    q_a = y(3*N+1:4*N-1);
    zeroq = true;
    for b = 1:N-1
       if abs(q_d(b)) > 10^-5 || abs(q_a(b)) > 10^-5 || q_p > 0 || q_bpp > 0
           zeroq = false;
       end
    end
    if strcmp(STATE,'flow') && zeroq
        STATE = 'flow';
    elseif strcmp(STATE,'zero') && q_p > 0
        STATE = 'build';
    elseif strcmp(STATE,'build') && y(end) > DP0
        STATE = 'flow';
    end
    
    %simulate model
    y = ode4(@fExtendedModel,[0 dT],y);
    y = y';
    y = y(:,end);
    y(4*N+1) = p1_temp;
    y(4*N+2) = p2_temp;
    YExtended(:,i) = y;
    
    if (mod(i,100) == 0)
        
        i*dT/simPoints
    end
    
    
end
%% plot
[pp pc pb] = findP(YExtended,N,dT);
subplot(2,2,1);
% plot(t,pbArr,'k');
plot(pb*10^-5);
hold on
plot(LOG.p_dh1(interval),'k--');
legend('Simulated');
xlabel('time [s]');
ylabel('p_b_i_t [bar]');
subplot(2,2,2);
plot(pc*10^-5);
hold on
plot(LOG.p_c(interval),'k--');
legend('Simulated','UllRigg Data');
xlabel('time [s]');
ylabel('p_c [bar]');
subplot(2,2,3);
plot(controlZC(1:20:end));
hold on
plot(z_choke,'k--');
legend('Simulated','UllRigg Data');
xlabel('time [s]');
ylabel('u_c [%]');
subplot(2,2,4);
plot(h_b,'k');
xlabel('time [s]');
ylabel('x_d [m]');
