function [Ydot] = fExtendedModel(t,y) 
% Implements the dynamic model of the well hydraulic system. Developed in
% the fall project, and contains a lot more detail than that used in the
% master thesis.

g     = 9.81;                     % [m/s2]    Gravitational acceleration
K_c   = 538;                      % [l/min/sqrt(bar)] Choke flow constant
p_0   = 10^5;                        % [Pa]     Pressure outside the system
tau0  = 11.7027;%6.82;

global z_c xDot rho_in beta_in q_p p_c rho_change p1_temp p2_temp pos_front_last w_dr q_bpp v_d geometry STATE steadyState q_c;

% Parameter added from the PDE block
N = 15;                            %  # cells in both drill string and annulus
l = geometry.l_d(1)*N/(N-0.5);
dh_a = geometry.dh_a;
dh_d = geometry.dh_d;
di = sum(geometry.D_dI)/N;
do = sum(geometry.D_dO)/N;
A_d = sum(geometry.A_d)/N;
A_a = sum(geometry.A_a)/N;
d_a = sqrt(4*A_a/pi+do^2);
% di = 0.1195;% sum(geometry.D_dI)/N;
% do = 0.1372; % sum(geometry.D_dO)/N;
% A_d = 0.0098; %sum(geometry.A_d)/N;
% A_a = 0.0269; %sum(geometry.A_a)/N;
% d_a = 0.2304; %sqrt(4*A_a/pi+do^2);
kappa = do/d_a;
mju = 0.1257;%0.4207;%0.3657;
beta = 17250*10^5;
a_an = 0.49;
b_an = 0.99;
rho_old = 1420;
rho_new = 1500;
dt = 0.05;
Re_crit = 1654;
y1 = 16/Re_crit;
y2 = -1/(16*log10(5.74/15000^0.9));
a = (y2-y1)/(15000-Re_crit);
b = y1 - a*Re_crit;


p1d = 1.2674e+08;
p2d = 1.6273e+05;
p1a = 5.0697e+07;
p2a = 5.0909e+04;

Z_c = [0 4 9 10 15 20 30 40 50 70]';
G_c = 1/700*[0 1 35 45 80 125 200 300 400 700]';
g_c=@(z) interp1( Z_c, G_c, max(Z_c(1), min(z, Z_c(end))) );

    function F = FricA(v,i)
        % F = (sign(v)*p2a+p1a*v*A_a)/N;
        if abs(v) < 10^-4
            f = 0;
        else
            K = mju*(a_an+b_an);
            Re = rho_a(i)*abs(v)*(d_a-do)/K;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
        end
        F = sign(v)*p2a/(N-0.5) + f*rho_a(i)*v*abs(v)*l*(d_a/2-do/2)/(d_a^2/4-do^2/4);
        if(w_dr ~= 0)
             deltah = (dh_a(i)+dh_a(i+1))/2;
            d_hydr = d_a-do;
             dp = p_a(i) - p_a(i+1) - rho_a(i)*g*deltah;
             tau = dp*(d_a^2/4-do^2/4)/(2*l*(d_a/2+do/2));
%            tau = 1;
            Ta = do*d_hydr^3*rho_a(i)^2*w_dr^2/(16*mju);
            e = 1/d_hydr;
            Re_eff = 8*rho_a(i)*v^2/tau;
            PRL = 0.26*(13.5+tau0/(rho_a(i)*v^2))^0.428*e^0.158*Ta^0.07*Re_eff^0.042*kappa*(1/kappa-1)^-0.0152;
            F = F*PRL;
            %Ta^0.0319
        end
    end

    function F = FricChangeA(v,rho)
        if abs(v) < 10^-4
            f = 0;
        else
            K = mju*(a_an+b_an);
            Re = rho*abs(v)*(d_a-do)/K;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
        end
        F = sign(v)*p2a/(N-0.5) + f*rho*v*abs(v)*l*(d_a/2-do/2)/(d_a^2/4-do^2/4);
    end
    function F = FricD(v,i)
        % F = (sign(v)*p2d+p1d*v*A_d)/(2*N);
        if abs(v) < 10^-4
            f = 0;
        else
            Re = rho_d(i)*abs(v)*di/mju;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
           % f = 16/Re;
        end
        F = sign(v)*p2a/(N-0.5) + 2*f*rho_d(i)*v*abs(v)*l/di;
        if w_dr > 0 && abs(v) > 10^-4
            d_e = di;
            Re_star = rho_d(i)*v*d_e/mju;
            Rew = rho_d(i)*d_e*w_dr*d_e/(2*mju);
            f = 29*Rew^0.23/(Re_star*4);
            F =sign(v)*p2a/(N-0.5)+ 2*f*rho_d(i)*v*abs(v)*l/d_e;
            % Re^0.16
        end
    end

    function F = FricChangeD(v,rho)
        if abs(v) < 10^-4
            f = 0;
        else
            Re = rho*v*di/mju;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
        end
        F = sign(v)*p2a/(N-0.5) + 2*f*rho*v*abs(v)*l/di;
    end

    function F = FricBit(v)
        %F = (p2d + p1d*q_bit)/2;
        F = p2d-p2a + 10*10^7*q_bit;%7.6751*10^7*q_bit;
    end


switch STATE
    
    case 'flow'
        rho_change_annulus = false;
        
        p_d = y(1:N);
        q_d = y(N+1:2*N-1);
        q_bit = y(2*N);
        p_a = y(2*N+1:3*N);
        q_a = y(3*N+1:4*N-1);
        x_f = y(4*N);
        p1 = y(4*N+1);
        p2 = y(4*N+2);
        q_f = y(4*N+3);
        x_b = y(4*N+4);
        p1_temp = p1;
        p2_temp = p2;
        
        rho_d = rho_new*ones(1,N);
        rho_a = rho_old*ones(1,N);
        
        pos_front = max(ceil(x_f/l),1);
        
        for i = pos_front:N
            rho_d(i) = rho_old;
        end
        
        if (pos_front > N && pos_front < 2*N+1)
            pos_front = pos_front - N;
            rho_change = false;
            rho_change_annulus = true;
            for i = 1:pos_front
                rho_a(i) = rho_new;
            end
        elseif pos_front >= 2*N+1
            rho_change_annulus = false;
            rho_change = false;
            rho_a = rho_new*ones(N,1);
        end
        
        p_dDot = xDot(1:N);
        q_dDot = xDot(N+1:2*N-1);
        q_bitDot = xDot(2*N);
        p_aDot = xDot(2*N+1:3*N);
        q_aDot = xDot(3*N+1:4*N-1);
        x_fDot = xDot(end);
        
        q_in = q_p;
        
        v = q_a(N-1)/(A_a);
        if rho_change_annulus
            p_c = p_a(N) - rho_old*g*dh_a(N)/2 - FricChangeA(v,rho_old)/2;
        else
            p_c = p_a(N) - rho_a(N)*g*dh_a(N)/2 - FricA(v,N-1)/2;    
        end
        if p_c < p_0
            p_c = p_0;
            q_out = 0;
        else
            if pos_front >= 2*N+1
                rho = rho_new;
            else
                rho = rho_old;
            end
            q_out = K_c*g_c(z_c)*sqrt(1400/rho)*sqrt((p_c-p_0)*10^-5)*0.001/60;    %flow through choke
        end
        q_c = q_out;
        
        v_bit = q_bit/(A_d);
        
        
        l1 = mod(x_f,l);
        if rho_change_annulus && pos_front == 1 && l1 < l/2
            rho_a0 = (l1*rho_new+(l/2-l1)*rho_old)/(l/2);
            M_a = rho_a0*l/(A_a*2);
        else
            rho_a0 = rho_a(1);
            M_a = rho_a(1)*l/(A_a*2);
        end
        if rho_change && pos_front == N && l1 > l/2
            rho_d0 = ((l1-l/2)*rho_new + (l-l1)*rho_old)/(l/2);
            M_d = rho_d0*l/(A_d*2);
        else
            rho_d0 = rho_d(N);
            M_d = rho_d(N)*l/(A_d*2);
        end
       % pos_bit = max(ceil(x_b/l),1);     
        %p_temp_bit = p_a(pos_bit) - rho_a(pos_bit)*g*x_b - FricA(q_a(pos_bit)/A_a,pos_bit)*x_b/l;
        if(v_bit > 0)
            q_bitDot = (1/(M_a+M_d))*(p_d(N)-p_a(1) - FricBit(v_bit)+(rho_d0-rho_a0)*g*dh_d(N)/2);
        else
            q_bit = 0;
            q_bitDot = max(0,(1/(M_a+M_d))*(p_d(N)-p_a(1) - FricBit(v_bit)+(rho_d0-rho_a0)*g*dh_d(N)/2));
        end
        

        q_d(N) = q_bit;
        q_a(N) = q_out;
        
        no_change_start = false;
        no_change_end = false;
        
        pos_front_last = pos_front;
        
        
        
        p_dDot(1) = beta/(A_d*l)*(q_in-q_d(1));
        p_dDot(N) = beta/(A_d*l)*(q_d(N-1)-q_bit);
        
        for k = 2:N-1
            p_dDot(k) = beta/(A_d*l)*(q_d(k-1)-q_d(k));
        end
        
        for k = 1:N-1
            v = q_d(k)/(A_d);        
            dh = (dh_d(k)+dh_d(k+1))/2;
            q_dDot(k) = (A_d/(l*rho_d(k)))*(p_d(k)-p_d(k+1))-FricD(v,k)*(A_d/(l*rho_d(k))) + g*A_d*dh/l;    
        end
        
        
        p_aDot(1) = beta/(A_a*l)*(q_bit-q_a(1));
        p_aDot(N) = beta/(A_a*l)*(q_a(N-1)-q_out + q_bpp);
        
        for k = 2:N-1
            p_aDot(k) = beta/(A_a*l)*(q_a(k-1)-q_a(k));
        end
        
        for k = 1:N-1
            v = q_a(k)/(A_a);
            dh = (dh_a(k)+dh_a(k+1))/2;
            q_aDot(k) = (A_a/(l*rho_a(k)))*(p_a(k)-p_a(k+1))-FricA(v,k)*(A_a/(l*rho_a(k))) - g*A_a*dh/l;
        end
        
        if rho_change
            x_fDot =  q_f/(A_d);
            l1 = mod(x_f,l);
            if l1 < 2*q_f*dt/A_d
                l1 = 2*q_f*dt/A_d;
                l2 = l-l1;
                no_change_start = true;
                if pos_front == 1
                    q = q_p;
                else
                    q = q_d(pos_front-1);
                end
                rho = (rho_new*l1 + rho_old*l2)/l;
                p2 = p_d(pos_front) + (dh_d(pos_front)/l)*rho_old*g*(l-l2)/2 - FricChangeD(q/A_d,rho_old)*(l-l2)/(2*l);
                p2_temp = p2;
                p1 = p2 - rho*g*dh_d(pos_front)/2 + FricChangeD(q/A_d,rho)/(2);
                p1_temp = p1;
            end
            l2 = l-l1;
            if l2 < 2*q_f*dt/A_d
                l2 = 2*q_f*dt/A_d;
                l1 = l-l2;
                no_change_end = true;
            end
            q = 0;
            if pos_front ~= 1
                l_bar = (l+l1)/2;
                dh = dh_d(pos_front-1)/2 + dh_d(pos_front)*l1/(2*l);
                q_dDot(pos_front-1) = A_d/(l_bar*rho_new)*(p_d(pos_front-1)-p1) - FricChangeD(q_d(pos_front-1)/A_d,rho_new)*A_d/(rho_new*l) + g*A_d*dh/l_bar;
                q = q_d(pos_front-1);
            else
                q = q_p;
            end
            if no_change_end || no_change_start
                p1_dot = 0;
            else
                F = FricChangeD(q_f/A_d,rho_new);
                p1_dot = beta/(l1*A_d)*(q-q_f) + q_f*rho_new*g/(2*A_d)*(dh_d(pos_front)/l) - F*q_f/(2*l*A_d);
            end
            rho = (rho_old*l2+rho_new*l1)/l;
            q_fDot = 2*A_d/(l*rho)*(p1-p2) - FricChangeD(q_f/A_d,rho)*A_d/(rho*l) + g*A_d*dh_d(pos_front)/l;
            if pos_front == N
                q = q_bit;
            else
                l_bar = (l+l2)/2;
                dh = dh_d(pos_front)*l2/(2*l) + dh_d(pos_front+1)/2;
                q_dDot(pos_front) = A_d/(l_bar*rho_old)*(p2-p_d(pos_front+1)) - FricChangeD(q_d(pos_front)/A_d,rho_old)*A_d/(l*rho_old) + g*A_d*dh/l_bar;
                q = q_d(pos_front);
            end
            if no_change_end || no_change_start
                p2_dot = 0;
            else
                p2_dot = beta/(l2*A_d)*(q_f-q) + q_f*rho_old*g/(2*A_d)*(dh_d(pos_front)/l) - FricChangeD(q_f/A_d,rho_old)*q_f/(2*l*A_d);
            end
            p_dDot(pos_front) = ((p2*l2+p1*l1)/l - p_d(pos_front))/dt;
            
        elseif rho_change_annulus
            x_fDot =  q_f/(A_a);
            l1 = mod(x_f,l);
            if l1 < 5*q_f*dt/A_a
                l1 = 5*q_f*dt/A_a;
                l2 = l-l1;
                no_change_start = true;
                if pos_front == 1
                    q = q_bit;
                else
                    q = q_a(pos_front-1);
                end
                rho = (rho_new*l1 + rho_old*l2)/l;
                p2 = p_a(pos_front) - (dh_a(pos_front)/l)*rho_old*g*(l-l2)/2 - FricChangeA(q/A_a,rho_old)*(l-l2)/(2*l);
                p2_temp = p2;
                p1 = p2 + rho*g*dh_a(pos_front)/2 + FricChangeA(q/A_a,rho)/2;
                p1_temp = p1;
            end
            l2 = l-l1;
            if l2 < 5*q_f*dt/A_a
                l2 = 5*q_f*dt/A_a;
                l1 = l-l2;
                no_change_end = true;
            end
            q = 0;
            if pos_front ~= 1
                l_bar = (l+l1)/2;
                dh = dh_a(pos_front-1)/2 + dh_a(pos_front)*l1/(2*l);
                q_aDot(pos_front-1) = A_a/(l_bar*rho_new)*(p_a(pos_front-1)-p1) - FricChangeA(q_a(pos_front-1)/A_a,rho_new)*A_a/(rho_new*l) - g*A_a*dh/l_bar;
                q = q_a(pos_front-1);
            else
                q = q_bit;
            end
            if no_change_end || no_change_start
                p1_dot = 0;
            else
                p1_dot = beta/(l1*A_a)*(q-q_f) - q_f*rho_new*g/(2*A_a)*(dh_a(pos_front)/l) - FricChangeA(q_f/A_a,rho_new)*q_f/(2*l*A_a);
            end
            rho = (rho_old*l2+rho_new*l1)/l;
            q_fDot = 2*A_a/(l*rho)*(p1-p2) - FricChangeA(q_f/A_a,rho)*A_a/(l*rho) - g*A_a*dh_a(pos_front)/l;
            if pos_front == N
                q = q_out;
            else
                l_bar = (l+l2)/2;
                dh = dh_a(pos_front)*l2/(2*l) + dh_a(pos_front+1)/2;
                q_aDot(pos_front) = A_a/(l_bar*rho_old)*(p2-p_a(pos_front+1)) - FricChangeA(q_a(pos_front)/A_a,rho_old)*A_a/(l*rho_old) - g*A_a*dh/l_bar;
                q = q_a(pos_front);
            end
            if no_change_end
                p2_dot = 0;
            elseif  no_change_start
                p2_dot = 0;
            else
                p2_dot = beta/(l2*A_a)*(q_f-q) - q_f*rho_old*g/(2*A_a)*(dh_a(pos_front)/l) - FricChangeA(q_f/A_a,rho_old)*q_f/(2*l*A_a);
            end
            
            p_aDot(pos_front) = ((p2*l2+p1*l1)/l - p_a(pos_front))/dt;
        else
            x_fDot = 0;
            q_fDot = 0;
            p1_dot = 0;
            p2_dot = 0;
        end
        
        if v_d ~= 0
            x_bDot = v_d;
            pos_bit = max(ceil(x_b/l),1);
            if pos_bit == 1
                q_inPos = q_bit;
                q_outPos = q_a(pos_bit);
            elseif pos_bit == N
                q_inPos = q_a(pos_bit-1) + q_bpp;
                q_outPos = q_out;
            else
                q_inPos = q_bit + q_a(pos_bit-1);
                q_outPos = q_a(pos_bit);
            end
            p_aDot(pos_bit) = beta/(A_a*(l-x_b)+x_b*(A_a+A_d))*(q_inPos - q_outPos - v_d*(pi*do^2)/4);
        else
            x_bDot = 0;
        end
        Ydot = [p_dDot; q_dDot; q_bitDot; p_aDot; q_aDot; x_fDot; p1_dot; p2_dot; q_fDot; x_bDot; 0];
        if max(abs(p_dDot)) < 10 && max(abs(p_aDot)) < 10 && max(abs(q_aDot)) < 10^-10 && max(abs(q_dDot)) < 10^-10
           steadyState = true; 
        else
            steadyState = false;
        end
    case 'zero'
        Ydot = zeros(4*N+5,1);
        
    case 'build'
        Ydot = zeros(4*N+5,1);
        Ydot(end) = beta/(A_d*N*l)*(q_p);
        
end


end