function [p_p p_c p_bit] = findP(Yex,N,dT)
% Returns the time series for the pump, choke and down hole pressures based
% on the pressures and flows in the corresponding control volumes
p_pumpArr = Yex(1,50:1/dT:end);
p_chokeArr = Yex(3*N,50:1/dT:end);
p_bitArr = Yex(2*N+1,50:1/dT:end);

qp = Yex(N+1,50:20:end);
qc = Yex(4*N-1,50:20:end);
qb = Yex(2*N,50:20:end);

w_drill = zeros(size(p_pumpArr));
p_d = Yex(1:N,50:1/dT:end);
p_a = Yex(2*N+1:3*N,50:1/dT:end);
g = 9.81;
a_an = 0.49;
b_an = 0.99;
mju = 0.1257;
tau0 = 11.7027;%6.82;


p2d = 1.6273e+05;
p2a = 5.0909e+04;

Cin2m = 2.54e-2;
P.V_a0  = 79.9;                     % [m3]      Annulus volume without drill string
P.L     = 1990.99;                  % [m]       Total length of well
P.L_p(1) = 1957.36;                 % [m]       Length with 5" drill string
P.L_p(2) = P.L-P.L_p(1);            % [m]       Length with 5.5" drill string
P.D_dI(1) = 4.2772*Cin2m;           % [m]       Inner diameter 5" drill string
P.D_dI(2) = 4.8125*Cin2m;           % [m]       Inner diameter 5.5"
P.D_dO(1) = 5.0*Cin2m;              % [m]       Outer diameter 5"
P.D_dO(2) = 5.5*Cin2m;              % [m]       Outer diameter 5.5"
P.N = N;
geometry = findAreasAndVolumes(P);

l = geometry.l_d(1);
dh_a = geometry.dh_a;
dh_d = geometry.dh_d;

di = sum(geometry.D_dI)/N;
do = sum(geometry.D_dO)/N;
A_d = sum(geometry.A_d)/N;
A_a = sum(geometry.A_a)/N;
d_a = sqrt(4*A_a/pi+do^2);
kappa = do/d_a;

Re_crit = 1654;
y1 = 16/Re_crit;
y2 = -1/(16*log10(5.74/15000^0.9));
a = (y2-y1)/(15000-Re_crit);
b = y1 - a*Re_crit;


rho = 1420;
p_p = zeros(size(p_pumpArr));
p_c = zeros(size(p_chokeArr));
p_bit = zeros(size(p_chokeArr));





for i = 1:length(p_pumpArr)
   p_p(i) =  p_pumpArr(i) - rho*9.81*dh_d(1)/2 + FricD(qp(i)/(A_d),1,i)/2;
   p_c(i) =  p_chokeArr(i) - rho*9.81*dh_a(N)/2 - FricA(qc(i)/(A_a),4,i)/2;
   p_bit(i) = p_bitArr(i) + rho*9.91*dh_a(1)/2 + FricBit(qb(i))/2;
    
end

    function F = FricA(v,i,k)
       % F = (p2a+p1a*v*A_a)/N;
        if abs(v) < 10^-4
            f = 0;
        else
            K = mju*(a_an+b_an);
            Re = rho*abs(v)*(d_a-do)/K;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
        end
        F = sign(v)*p2a/N + f*rho*v*abs(v)*l*(d_a/2-do/2)/(d_a^2/4-do^2/4);
        if(w_drill(k) ~= 0)
            deltah = (dh_a(i)+dh_a(i+1))/2;
            d_hydr = d_a-do;
            dp = p_a(i,k) - p_a(i+1,k) - rho*g*deltah;
            tau = dp*(d_a^2/4-do^2/4)/(2*l*(d_a/2+do/2));
            Ta = do*d_hydr^3*rho^2*w_drill(k)^2/(16*mju);
            e = 1/d_hydr;
            Re_eff = 8*rho^2/tau;
            PRL = 0.26*(13.5+tau0/(rho*v^2))^0.428*e^0.158*Ta^0.07*Re_eff^0.042*kappa*(1/kappa-1)^-0.0152;
            F = F*PRL;
        end   
    end


    function F = FricD(v,i,k)
        %F = (p2d+p1d*v*A_d)/(2*N);
        if abs(v) < 10^-4
            f = 0;
        else
            Re = rho*v*di/mju;
            if Re < Re_crit
                f = 16/Re;
            else
                f = a*Re+b;
            end
           % f = 16/Re;
        end
        F = sign(v)*p2a/(N) + 2*f*rho*v*abs(v)*l/di;
        if w_drill(k) > 0
            d_e = di;
            Re_star = rho*v*d_e/mju;
            Rew = rho*d_e*w_drill(k)*d_e/(2*mju);
            f = 29*Rew^0.23/(Re_star*4);
            F =sign(v)*p2a/(N)+ 2*f*rho*v*abs(v)*l/d_e;
        end
    end

    function F = FricBit(q_bit)
        %F = (p2d + p1d*q_bit)/2;
        F = p2d-p2a + 10*10^7*q_bit;%7.6751*10^7*q_bit;
    end


end