function x0 = initSystem(q0,N, geometry, pc0)
%Initializes the pressure and flow states
g     = 9.81;                     % [m/s2]    Gravitational acceleration
K_c   = 538;                      % [l/min/sqrt(bar)] Choke flow constant
Re_crit = 1654;
y1 = 16/Re_crit;
y2 = -1/(16*log10(5.74/15000^0.9));
a = (y2-y1)/(15000-Re_crit);
b = y1 - a*Re_crit;


% Parameter added from the PDE block
p2d = 1.6273e+05;
p2a = 5.0909e+04;

mju = 0.1257;

rho = 1420;
l = geometry.l_d(1)*N/(N-0.5);
dh_a = geometry.dh_a;
dh_d = geometry.dh_d;
% di = 0.1195;% sum(geometry.D_dI)/N;
% do = 0.1372; % sum(geometry.D_dO)/N;
% A_d = 0.0098; %sum(geometry.A_d)/N;
% A_a = 0.0269; %sum(geometry.A_a)/N;
% d_a = 0.2304; %sqrt(4*A_a/pi+do^2);
di = sum(geometry.D_dI)/N;
do = sum(geometry.D_dO)/N;
A_d = sum(geometry.A_d)/N;
A_a = sum(geometry.A_a)/N;
d_a = sqrt(4*A_a/pi+do^2);
a_an = 0.49;
b_an = 0.99;

p_c = pc0;
% p_c = (((q0*60/0.001)/(K_c*1*sqrt(1400/1420)))^2+1)*10^5;

p0_d = zeros(1,N);
p0_a = zeros(1,N);


v = q0/A_a;
if abs(v) < 10^-4
    f = 0;
else
    K = mju*(a_an+b_an);
    Re = rho*abs(v)*(d_a-do)/K;
    if Re < Re_crit
        f = 16/Re;
    else
        f = a*Re+b;
    end
end
F = sign(v)*p2a/(N-0.5) + f*rho*v*abs(v)*l*(d_a/2-do/2)/(d_a^2/4-do^2/4);
%F = (p2a+p1a*q0)/N;

p0_a(N) = p_c + rho*g*dh_a(N)/2 + F/2;

for i = 1:N-1
    dh = dh_a(N-i)/2 + dh_a(N-i+1)/2;
    p0_a(N-i) =  p0_a(N-i+1) + rho*g*dh + F;
end

p0_d(N) = p0_a(1) + p2d-p2a + 10*10^7*q0;

for i = 1:N-1
    v = q0/A_d;
    if abs(v) < 10^-4
        f = 0;
    else
        Re = rho*v*di/mju;
        if Re < Re_crit
            f = 16/Re;
        else
            f = a*Re+b;
        end
    end
    F = sign(v)*p2a/(N-0.5) + 2*f*rho*v*abs(v)*l/di;
    dh = dh_d(N-i)/2 + dh_d(N-i+1)/2;
    p0_d(N-i) = p0_d(N-i+1) - rho*g*dh + F;
end


q0_d = q0*ones(1,N-1);
q0_a = q0*ones(1,N-1);
q0_b = q0;
xf0 = 0;
xb0 = 19;
qf0 = q0;
p20 = p0_d(1);
p10 = p0_d(1) - rho*g*l/2 + F/2;

x0 = [p0_d, q0_d, q0_b, p0_a, q0_a, xf0, p10, p20, qf0, xb0];

end